@@ -31,9 +31,10 @@
 #include "strv.h"
 #include "path-util.h"
 #include "selinux-util.h"
+#include "missing.h"
 
 #ifdef HAVE_SELINUX
-DEFINE_TRIVIAL_CLEANUP_FUNC(security_context_t, freecon);
+DEFINE_TRIVIAL_CLEANUP_FUNC(char *, freecon);
 DEFINE_TRIVIAL_CLEANUP_FUNC(context_t, context_free);
 
 #define _cleanup_security_context_free_ _cleanup_(freeconp)
@@ -67,7 +68,12 @@ int mac_selinux_init(const char *prefix) {
 
 #ifdef HAVE_SELINUX
         usec_t before_timestamp, after_timestamp;
+
+#ifdef HAVE_MALLINFO2
+        struct mallinfo2 before_mallinfo, after_mallinfo;
+#elif HAVE_MALLINFO
         struct mallinfo before_mallinfo, after_mallinfo;
+#endif
 
         if (!mac_selinux_use())
                 return 0;
@@ -75,7 +81,12 @@ int mac_selinux_init(const char *prefix) {
         if (label_hnd)
                 return 0;
 
+#ifdef HAVE_MALLINFO2
+        before_mallinfo = mallinfo2();
+#elif HAVE_MALLINFO
         before_mallinfo = mallinfo();
+#endif
+
         before_timestamp = now(CLOCK_MONOTONIC);
 
         if (prefix) {
@@ -92,16 +103,29 @@ int mac_selinux_init(const char *prefix) {
                 r = security_getenforce() == 1 ? -errno : 0;
         } else  {
                 char timespan[FORMAT_TIMESPAN_MAX];
+
+#if defined(HAVE_MALLINFO)|defined(HAVE_MALLINFO2)
                 int l;
+#endif
 
                 after_timestamp = now(CLOCK_MONOTONIC);
-                after_mallinfo = mallinfo();
 
+#ifdef HAVE_MALLINFO2
+                after_mallinfo = mallinfo2();
+#elif HAVE_MALLINFO
+                after_mallinfo = mallinfo();
+#endif
+
+#if defined(HAVE_MALLINFO)|defined(HAVE_MALLINFO2)
                 l = after_mallinfo.uordblks > before_mallinfo.uordblks ? after_mallinfo.uordblks - before_mallinfo.uordblks : 0;
 
                 log_debug("Successfully loaded SELinux database in %s, size on heap is %iK.",
                           format_timespan(timespan, sizeof(timespan), after_timestamp - before_timestamp, 0),
                           (l+1023)/1024);
+#else
+                log_debug("Successfully loaded SELinux database in %s",
+                          format_timespan(timespan, sizeof(timespan), after_timestamp - before_timestamp, 0));
+#endif
         }
 #endif
 
@@ -133,7 +157,7 @@ int mac_selinux_fix(const char *path, bool ignore_enoent, bool ignore_erofs) {
 
         r = lstat(path, &st);
         if (r >= 0) {
-                _cleanup_security_context_free_ security_context_t fcon = NULL;
+                _cleanup_security_context_free_ char * fcon = NULL;
 
                 r = selabel_lookup_raw(label_hnd, &fcon, path, st.st_mode);
 
@@ -176,7 +200,7 @@ int mac_selinux_apply(const char *path, const char *label) {
         if (!mac_selinux_use())
                 return 0;
 
-        if (setfilecon(path, (security_context_t) label) < 0) {
+        if (setfilecon(path, label) < 0) {
                 log_enforcing("Failed to set SELinux security context %s on path %s: %m", label, path);
                 if (security_getenforce() == 1)
                         return -errno;
@@ -189,7 +213,7 @@ int mac_selinux_get_create_label_from_exe(const char *exe, char **label) {
         int r = -EOPNOTSUPP;
 
 #ifdef HAVE_SELINUX
-        _cleanup_security_context_free_ security_context_t mycon = NULL, fcon = NULL;
+        _cleanup_security_context_free_ char *mycon = NULL, *fcon = NULL;
         security_class_t sclass;
 
         assert(exe);
@@ -207,7 +231,7 @@ int mac_selinux_get_create_label_from_exe(const char *exe, char **label) {
                 return -errno;
 
         sclass = string_to_security_class("process");
-        r = security_compute_create(mycon, fcon, sclass, (security_context_t *) label);
+        r = security_compute_create(mycon, fcon, sclass, label);
         if (r < 0)
                 return -errno;
 #endif
@@ -236,7 +260,7 @@ int mac_selinux_get_child_mls_label(int socket_fd, const char *exe, const char *
         int r = -EOPNOTSUPP;
 
 #ifdef HAVE_SELINUX
-        _cleanup_security_context_free_ security_context_t mycon = NULL, peercon = NULL, fcon = NULL;
+        _cleanup_security_context_free_ char *mycon = NULL, *peercon = NULL, *fcon = NULL;
         _cleanup_context_free_ context_t pcon = NULL, bcon = NULL;
         security_class_t sclass;
         const char *range = NULL;
@@ -286,7 +310,7 @@ int mac_selinux_get_child_mls_label(int socket_fd, const char *exe, const char *
                 return -ENOMEM;
 
         sclass = string_to_security_class("process");
-        r = security_compute_create(mycon, fcon, sclass, (security_context_t *) label);
+        r = security_compute_create(mycon, fcon, sclass, label);
         if (r < 0)
                 return -errno;
 #endif
@@ -300,7 +324,7 @@ void mac_selinux_free(char *label) {
         if (!mac_selinux_use())
                 return;
 
-        freecon((security_context_t) label);
+        freecon(label);
 #endif
 }
 
@@ -308,7 +332,7 @@ int mac_selinux_create_file_prepare(const char *path, mode_t mode) {
         int r = 0;
 
 #ifdef HAVE_SELINUX
-        _cleanup_security_context_free_ security_context_t filecon = NULL;
+        _cleanup_security_context_free_ char *filecon = NULL;
 
         assert(path);
 
@@ -368,7 +392,7 @@ int mac_selinux_create_socket_prepare(const char *label) {
 
         assert(label);
 
-        if (setsockcreatecon((security_context_t) label) < 0) {
+        if (setsockcreatecon(label) < 0) {
                 log_enforcing("Failed to set SELinux security context %s for sockets: %m", label);
 
                 if (security_getenforce() == 1)
@@ -396,7 +420,7 @@ int mac_selinux_mkdir(const char *path, mode_t mode) {
         /* Creates a directory and labels it according to the SELinux policy */
 
 #ifdef HAVE_SELINUX
-        _cleanup_security_context_free_ security_context_t fcon = NULL;
+        _cleanup_security_context_free_ char *fcon = NULL;
         int r;
 
         assert(path);
@@ -446,7 +470,7 @@ int mac_selinux_bind(int fd, const struct sockaddr *addr, socklen_t addrlen) {
         /* Binds a socket and label its file system object according to the SELinux policy */
 
 #ifdef HAVE_SELINUX
-        _cleanup_security_context_free_ security_context_t fcon = NULL;
+        _cleanup_security_context_free_ char *fcon = NULL;
         const struct sockaddr_un *un;
         char *path;
         int r;
