@@ -33,6 +33,7 @@
 #include <sys/socket.h>
 #include <sys/signalfd.h>
 #include <linux/sockios.h>
+#include <sys/sysmacros.h>
 
 #include "udev.h"
 
@@ -62,7 +63,9 @@ void udev_event_unref(struct udev_event *event) {
         free(event);
 }
 
-size_t udev_event_apply_format(struct udev_event *event, const char *src, char *dest, size_t size) {
+size_t udev_event_apply_format(struct udev_event *event,
+                               const char *src, char *dest, size_t size,
+                               bool replace_whitespace) {
         struct udev_device *dev = event->dev;
         enum subst_type {
                 SUBST_UNKNOWN,
@@ -117,8 +120,10 @@ size_t udev_event_apply_format(struct udev_event *event, const char *src, char *
 
         for (;;) {
                 enum subst_type type = SUBST_UNKNOWN;
-                char attrbuf[UTIL_PATH_SIZE];
-                char *attr = NULL;
+                char attrbuf[UTIL_PATH_SIZE], sbuf[UTIL_PATH_SIZE];
+                char *attr = NULL, *_s;
+                size_t _l;
+                bool replws = replace_whitespace;
 
                 while (from[0] != '\0') {
                         if (from[0] == '$') {
@@ -187,6 +192,19 @@ subst:
                         attr = NULL;
                 }
 
+                /* result subst handles space as field separator */
+                if (type == SUBST_RESULT)
+                        replws = false;
+
+                if (replws) {
+                        /* store dest string ptr and remaining len */
+                        _s = s;
+                        _l = l;
+                        /* temporarily use sbuf */
+                        s = sbuf;
+                        l = UTIL_PATH_SIZE;
+                }
+
                 switch (type) {
                 case SUBST_DEVPATH:
                         l = strpcpy(&s, l, udev_device_get_devpath(dev));
@@ -367,6 +385,20 @@ subst:
                         log_error("unknown substitution type=%i", type);
                         break;
                 }
+
+                /* replace whitespace in sbuf and copy to dest */
+                if (replws) {
+                        size_t tmplen = UTIL_PATH_SIZE - l;
+
+                        /* restore s and l to dest string values */
+                        s = _s;
+                        l = _l;
+
+                        /* copy ws-replaced value to s */
+                        tmplen = util_replace_whitespace(sbuf, s, MIN(tmplen, l));
+                        l -= tmplen;
+                        s += tmplen;
+                }
         }
 
 out:
@@ -650,10 +682,12 @@ int udev_build_argv(struct udev *udev, char *cmd, int *argc, char *argv[]) {
 
         pos = cmd;
         while (pos != NULL && pos[0] != '\0') {
-                if (pos[0] == '\'') {
-                        /* do not separate quotes */
+               if (IN_SET(pos[0], '\'', '"')) {
+                        /* do not separate quotes or double quotes */
+                        char delim[2] = { pos[0], '\0' };
+
                         pos++;
-                        argv[i] = strsep(&pos, "\'");
+                        argv[i] = strsep(&pos, delim);
                         if (pos != NULL)
                                 while (pos[0] == ' ')
                                         pos++;
@@ -771,17 +805,17 @@ out:
         return err;
 }
 
-static int rename_netif(struct udev_event *event) {
-        struct udev_device *dev = event->dev;
-        char name[IFNAMSIZ];
-        const char *oldname;
+#ifdef ENABLE_RULE_GENERATOR
+/* function to return the count of rules that assign NAME= to a value matching arg#2 , defined in udev-rules.c */
+int udev_rules_assigning_name_to(struct udev_rules *rules,const char *match_name);
+#endif
+
+static int rename_netif_dev_fromname_toname(struct udev_device *dev,const char *oldname,const char *name) {
         int r;
         int sk;
         struct ifreq ifr;
 
-        oldname = udev_device_get_sysname(dev);
-
-        strscpy(name, IFNAMSIZ, event->name);
+        log_debug("changing net interface name from '%s' to '%s'\n",oldname,name);
 
         sk = socket(PF_INET, SOCK_DGRAM, 0);
         if (sk < 0)
@@ -791,13 +825,63 @@ static int rename_netif(struct udev_event *event) {
         strscpy(ifr.ifr_name, IFNAMSIZ, oldname);
         strscpy(ifr.ifr_newname, IFNAMSIZ, name);
         r = ioctl(sk, SIOCSIFNAME, &ifr);
-        if (r < 0)
-                return log_error_errno(-errno, "Error changing net interface name '%s' to '%s': %m", oldname, name);
 
-        log_debug("renamed network interface '%s' to '%s'", oldname, name);
+#ifdef ENABLE_RULE_GENERATOR
+        int loop;
+    struct ifreq ifr_tmp;
+
+        if (r == 0) {
+                log_info("renamed network interface %s to %s\n", ifr.ifr_name, ifr.ifr_newname);
+                goto out;
+        }
+        /* keep trying if the destination interface name already exists */
+        log_debug("collision on rename of network interface %s to %s , retrying until timeout\n",
+                ifr.ifr_name, ifr.ifr_newname);
+
+        /* there has been a collision so rename my name to a temporal name, letting other one to rename to my name, freeying its name... */
+        memzero(&ifr_tmp, sizeof(struct ifreq));
+        strscpy(ifr_tmp.ifr_name, IFNAMSIZ, oldname);
+        snprintf(ifr_tmp.ifr_newname, IFNAMSIZ, "rename_%s", oldname);
+        r = ioctl(sk, SIOCSIFNAME, &ifr_tmp);
+                log_info("Temporarily renamed network interface %s to %s\n", ifr_tmp.ifr_name, ifr_tmp.ifr_newname);
+
+        /* we have changed our name so in subsequents tries i should rename my temporal name to the wanted one */
+        strscpy(ifr.ifr_name, IFNAMSIZ, ifr_tmp.ifr_newname);
+
+        r = -errno;
+        if (r != -EEXIST)
+                goto out;
+
+        /* wait a maximum of 90 seconds for our target to become available */
+        loop = 90 * 20;
+        while (loop--) {
+                const struct timespec duration = { 0, 1000 * 1000 * 1000 / 20 };
+
+                nanosleep(&duration, NULL);
+
+                r = ioctl(sk, SIOCSIFNAME, &ifr);
+                if (r == 0) {
+                        log_info("renamed network interface %s to %s\n", ifr.ifr_name, ifr.ifr_newname);
+                        break;
+                }
+                r = -errno;
+                if (r != -EEXIST)
+                        break;
+        }
+
+out:
+#endif
+        if (r < 0)
+                log_error_errno(-errno, "Error changing net interface name %s to %s: %m\n", ifr.ifr_name, ifr.ifr_newname);
+        else
+                log_debug("renamed network interface '%s' to '%s'", oldname, name);
 
         close(sk);
-        return 0;
+        return r;
+}
+
+static int rename_netif(struct udev_event *event) {
+        return rename_netif_dev_fromname_toname(event->dev,udev_device_get_sysname(event->dev),event->name);
 }
 
 void udev_event_execute_rules(struct udev_event *event,
@@ -843,6 +927,79 @@ void udev_event_execute_rules(struct udev_event *event,
                                           sigmask);
 
                 /* rename a new network interface, if needed */
+
+                /* ENABLE_RULE_GENERATOR conditional:
+                 * if this is a net iface, and it is an add event,
+                 * and as long as all of the following are FALSE:
+                 *  - no NAME target and the current name is not being used
+                 *  - there is a NAME target and it is the same as the current name
+                 *  - the rules can successfully be searched for the current name (not really part of the conditional)
+                 * the run the rename.
+                 *
+                 * note - udev_rules_assigning_name_to is run when event->name is NULL to ensure renames happen,
+                 * but also on its own to check if a temp-rename is necessary when event->name exists.
+                 *
+                 * A temp-rename is necessary when:
+                 * - there is no rule renaming the current iface but the current name IS used in some other rule
+                 * - there is a rule renaming the current iface,
+                 *   the current name IS used AND the target name != the current name
+                 */
+
+#ifdef ENABLE_RULE_GENERATOR
+                int r;
+                if (udev_device_get_ifindex(dev) > 0 && streq(udev_device_get_action(dev), "add") &&
+                    (event->name == NULL && (r=udev_rules_assigning_name_to(rules,udev_device_get_sysname(dev))) > 0 ||
+                    event->name != NULL && !streq(event->name, udev_device_get_sysname(dev)))) {
+                        char syspath[UTIL_PATH_SIZE];
+                        char *pos;
+                        char *finalifname = event->name;
+                        char newifname[IFNAMSIZ];
+
+                        /* r is the number of rules that assign a device with NAME= this sysname */
+                        if (r > 0 || (r=udev_rules_assigning_name_to(rules,udev_device_get_sysname(dev))) > 0) {
+                                /* have a conflict, rename to a temp name */
+                                char *newpos;
+                                int ifidnum;
+
+                                /* build the temporary iface name */
+                                strscpy(newifname, IFNAMSIZ, udev_device_get_sysname(dev));
+                                newpos=pos=&newifname[strcspn(newifname,"0123456789")];
+                                ifidnum=(int)strtol(pos,&newpos,10);
+                                *pos='\0';
+                                if (newpos > pos && *newpos == '\0') /* append new iface num to name */
+                                        /* use udev_device_get_ifindex(dev) as it is unique to every iface */
+                                        snprintf(pos,IFNAMSIZ+(newifname-pos), "%d", 128 - udev_device_get_ifindex(dev));
+
+                                /* note, r > 0, which will skip the post-rename stuff if no rename occurs */
+
+                                /* if sysname isn't already the tmpname (ie there is no numeric component), do the rename */
+                                if (!streq(newifname,udev_device_get_sysname(dev))) {
+                                        r = rename_netif_dev_fromname_toname(dev,udev_device_get_sysname(dev),newifname);
+                                        if (r == 0) {
+                                                finalifname = newifname;
+                                                log_debug("renamed netif to '%s' for collision avoidance\n", newifname);
+                                        } else {
+                                                log_error("could not rename netif to '%s' for collision avoidance\n",newifname);
+                                        }
+                                }
+                                /* rename it now to its final target if its not already there */
+                                if (event->name != NULL && !streq(event->name, newifname)) {
+                                        r = rename_netif_dev_fromname_toname(dev,newifname,event->name);
+                                        if (r == 0)
+                                                finalifname = event->name;
+                               }
+
+                        } else { /* no need to rename to a tempname first, do a regular direct rename to event->name */
+
+                                r = 1; /* skip the post-rename stuff if no rename occurs */
+                                if (!streq(event->name, udev_device_get_sysname(dev)))
+                                        r = rename_netif(event);
+                        }
+
+                        if (r == 0) {
+                                log_debug("renamed netif to '%s'\n", finalifname);
+                                r = udev_device_rename(dev, finalifname);
+#else
                 if (udev_device_get_ifindex(dev) > 0 && streq(udev_device_get_action(dev), "add") &&
                     event->name != NULL && !streq(event->name, udev_device_get_sysname(dev))) {
                         int r;
@@ -853,6 +1010,7 @@ void udev_event_execute_rules(struct udev_event *event,
                                                   udev_device_get_sysname(dev), event->name);
                         else {
                                 r = udev_device_rename(dev, event->name);
+#endif
                                 if (r < 0)
                                         log_warning_errno(r, "renamed interface '%d' from '%s' to '%s', but could not update udev_device: %m",
                                                           udev_device_get_ifindex(dev), udev_device_get_sysname(dev), event->name);
@@ -913,7 +1071,7 @@ void udev_event_execute_run(struct udev_event *event, usec_t timeout_usec, usec_
                 if (builtin_cmd < UDEV_BUILTIN_MAX) {
                         char command[UTIL_PATH_SIZE];
 
-                        udev_event_apply_format(event, cmd, command, sizeof(command));
+                        udev_event_apply_format(event, cmd, command, sizeof(command), false);
                         udev_builtin_run(event->dev, builtin_cmd, command, false);
                 } else {
                         char program[UTIL_PATH_SIZE];
@@ -924,7 +1082,7 @@ void udev_event_execute_run(struct udev_event *event, usec_t timeout_usec, usec_
                                 sleep(event->exec_delay);
                         }
 
-                        udev_event_apply_format(event, cmd, program, sizeof(program));
+                        udev_event_apply_format(event, cmd, program, sizeof(program), false);
                         envp = udev_device_get_properties_envp(event->dev);
                         udev_event_spawn(event, timeout_usec, timeout_warn_usec, program, envp, sigmask, NULL, 0);
                 }
